@@ -3,13 +3,14 @@ import time
 from dataclasses import dataclass
 import backoff  # type: ignore
 import requests
-from requests.exceptions import Timeout
+from requests.exceptions import Timeout, HTTPError, ConnectionError
 import spectacles.utils as utils
 from spectacles.types import JsonDict
 from spectacles.logger import GLOBAL_LOGGER as logger
 from spectacles.exceptions import SpectaclesException, LookerApiError
 
 TIMEOUT_SEC = 300
+BACKOFF_EXCEPTIONS = (Timeout, HTTPError, ConnectionError)
 
 
 @dataclass(frozen=True)  # Token is immutable
@@ -80,6 +81,7 @@ class LookerClient:
         self.api_version: float = api_version
         self.access_token: Optional[AccessToken] = None
         self.session: requests.Session = requests.Session()
+        self.workspace: str = "production"
 
         self.authenticate()
 
@@ -128,10 +130,17 @@ class LookerClient:
             f"using Looker API {self.api_version}"
         )
 
+    @backoff.on_exception(
+        backoff.expo,
+        BACKOFF_EXCEPTIONS,
+        max_tries=2,
+    )
     def request(self, method: str, url: str, *args, **kwargs) -> requests.Response:
         if self.access_token and self.access_token.expired:
             logger.debug("Looker API access token has expired, requesting a new one")
             self.authenticate()
+            if self.workspace == "dev":
+                self.update_workspace("dev")
         return self.session.request(method, url, *args, **kwargs)
 
     def get(self, url, *args, **kwargs) -> requests.Response:
@@ -229,6 +238,7 @@ class LookerClient:
                 ),
                 response=response,
             )
+        self.workspace = workspace
 
     def get_all_branches(self, project: str) -> List[str]:
         """Returns a list of git branches in the project repository.
@@ -565,7 +575,7 @@ class LookerClient:
 
         return response.json()
 
-    def get_lookml_models(self, fields: List = []) -> List[JsonDict]:
+    def get_lookml_models(self, fields: Optional[List] = None) -> List[JsonDict]:
         """Gets all models and explores from the LookmlModel endpoint.
 
         Returns:
@@ -573,6 +583,8 @@ class LookerClient:
 
         """
         logger.debug(f"Getting all models and explores from {self.base_url}")
+        if fields is None:
+            fields = []
 
         params = {}
         if fields:
@@ -629,9 +641,8 @@ class LookerClient:
 
         return response.json()["fields"]["dimensions"]
 
-    @backoff.on_exception(backoff.expo, (Timeout,), max_tries=2)
     def create_query(
-        self, model: str, explore: str, dimensions: List[str], fields: List = []
+        self, model: str, explore: str, dimensions: List[str], fields: List = None
     ) -> Dict:
         """Creates a Looker async query for one or more specified dimensions.
 
@@ -657,8 +668,10 @@ class LookerClient:
             "filter_expression": "1=2",
         }
 
-        params = {}
-        if fields:
+        params: Dict[str, list] = {}
+        if fields is None:
+            params["fields"] = []
+        else:
             params["fields"] = fields
 
         url = utils.compose_url(self.api_url, path=["queries"], params=params)
@@ -689,7 +702,6 @@ class LookerClient:
         )
         return result
 
-    @backoff.on_exception(backoff.expo, (Timeout,), max_tries=2)
     def create_query_task(self, query_id: int) -> str:
         """Runs a previously created query asynchronously and returns the query task ID.
 
@@ -791,7 +803,9 @@ class LookerClient:
     def content_validation(self) -> JsonDict:
         logger.debug("Validating all content in Looker")
         url = utils.compose_url(self.api_url, path=["content_validation"])
-        response = self.get(url=url, timeout=TIMEOUT_SEC)
+        response = self.get(
+            url=url, timeout=3600
+        )  # 1 hour timeout for content validation
 
         try:
             response.raise_for_status()
@@ -807,7 +821,26 @@ class LookerClient:
         result = response.json()
         return result
 
-    def all_folders(self, project: str) -> List[JsonDict]:
+    def lookml_validation(self, project) -> JsonDict:
+        logger.debug(f"Validating LookML for project '{project}'")
+        url = utils.compose_url(self.api_url, path=["projects", project, "validate"])
+        response = self.post(url=url, timeout=TIMEOUT_SEC)
+
+        try:
+            response.raise_for_status()
+        except requests.exceptions.HTTPError:
+            raise LookerApiError(
+                name="unable-to-validate-lookml",
+                title=f"Couldn't validate LookML in project {project}.",
+                status=response.status_code,
+                detail=("Failed to run the LookML validator. Please try again."),
+                response=response,
+            )
+
+        result = response.json()
+        return result
+
+    def all_folders(self) -> List[JsonDict]:
         logger.debug("Getting information about all folders")
         url = utils.compose_url(self.api_url, path=["folders"])
         response = self.get(url=url, timeout=TIMEOUT_SEC)
@@ -819,13 +852,47 @@ class LookerClient:
                 name="unable-to-get-folders",
                 title="Couldn't obtain project folders.",
                 status=response.status_code,
-                detail=(f"Failed to get all folders for project '{project}'."),
+                detail=("Failed to get all folders."),
                 response=response,
             )
 
         result = response.json()
         return result
 
+    @backoff.on_exception(backoff.expo, (Timeout,), max_tries=2)
+    def run_query(self, query_id: int) -> str:
+        """Returns the compiled SQL for a given query ID.
+
+        The corresponding Looker API endpoint allows us to run queries with a variety
+        of result formats, however we only use the `sql` result format, which doesn't
+        run the query but does return its compiled SQL.
+
+        If a Timeout exception is received, attempts to retry.
+
+        """
+        # Using old-style string formatting so that strings are formatted lazily
+        logger.debug("Retrieving the SQL for query ID %s", query_id)
+
+        url = utils.compose_url(self.api_url, path=["queries", query_id, "run", "sql"])
+        response = self.get(url=url, timeout=TIMEOUT_SEC)
+        try:
+            response.raise_for_status()
+        except requests.exceptions.HTTPError:
+            raise LookerApiError(
+                name="unable-to-retrieve-compiled-sql",
+                title="Couldn't retrieve compiled SQL.",
+                status=response.status_code,
+                detail=(
+                    f"Failed to retrieve compiled SQL for query '{query_id}'. "
+                    "Please try again."
+                ),
+                response=response,
+            )
+
+        result = response.text
+
+        return result
+
 
 class NullAuth(requests.auth.AuthBase):
     """A custom auth class which ensures requests does not override authorization
